% Implements weak sep test by Cherchye, L. T. Demuynck, B. De Rock and P. Hjertstrand (2015) 
% "Revealed preference tests for weak separability: an integer programming approach"
% Journal of Econometrics 186, 129-141.
%
% This procedure checks whether the weak separability conditions are satisfied for a given F
%
% NOTE: Requires TOMLAB/CPLEX. See http://tomopt.com/tomlab/products/cplex/

function pass = CDDH_F(p,x,sep,F)

[T K] = size(p);                           % size of data, T: #obs and K: #goods

E = sum((p.*x)')';                         % total expenditure

NUMG = (1:1:(1+1*(K-1)))';                 % sequence of numbers up to K

tol = 10^(-6);                             % tolerance level

% Note: We follow Nobibon, F.T., L. Cherchye, Y. Crama, T. Demuynck, B. De Rock, F. Spieksma (2013)
% "Revealed preference tests of collectively rational consumption behavior: formulations and algorithms" 
% (Available at SSRN: http://ssrn.com/abstract=2381956 or http://dx.doi.org/10.2139/ssrn.2381956) 
% who set the tolerance to 10^(-6).See pages 8, 10-11 and 18 in their paper
                                           
% SEPARABLE GROUP

r = p(:,sep);                              % prices in separable group
y = x(:,sep);                              % quantities in separable group

KSEP = size(r,2);                          % # goods in separable group

ESEP = sum((r.*y)')';                      % expenditure for separable group

ry = zeros(T,T);                           % cross-expenditure matrix for separable group
for i=1:T;
    for j=1:T;  
       ry(i,j) = sum((r(i,:).*(y(i,:)-y(j,:)))');
    end;
end;

ry = ry-F;

% NON-SEPARABLE GROUP

nsep = NUMG';
nsep(sep)=[];                               % goods in non-separable group

z = p(:,nsep);                              % prices in non-separable group
w = x(:,nsep);                              % quantities in non-separable group

KNSEP = size(z,2);                          % # goods in non-separable group

ENSEP = sum((z.*w)')';                      % expenditure for non-separable group

zw = zeros(T,T);                            % cross-expenditure matrix for non-separable group
for i=1:T;
    for j=1:T;  
       zw(i,j) = sum((z(i,:).*(w(i,:)-w(j,:)))');
    end;
end;

zw = zw-F;

% error message if dimension of groups are incorrect
if (KSEP+KNSEP ~= K); disp('NUMBER OF SEPARABLE AND NONSEPARABLE GOODS DOES NOT MATCH: PROBLEM!!!'); end; 

Avec = ENSEP+(1+1);    % large numbers

% Setting up MIP problem

M = 5*T*(T-1);            % # restrictions in MIP
NPAR = 3*T+T*(T-1);       % # parameters in MIP

A = zeros(M,NPAR);        % constraint matrix
b_L = -Inf*ones(M,1);     % lower bounds for constraints
b_U = zeros(M,1);         % upper bounds for constraints

% building constraint matrices/vectors

i = 0;
for t=1:T;
    for s=1:T;
        if (s ~= t);
            i = i + 1;
            
            % Sub-utility function:
            % V_t - V_s + l_s*r_s*(y_s-y_t) <= 0
            
            A(i,t) = 1;
            A(i,s) = -1;
            A(i,2*T+s) = ry(s,t);
            
            % First MIP inequality:
            % u_t - u_s - X_ts < 0
            
            A(T*(T-1)+i,T+t) = 1;
            A(T*(T-1)+i,T+s) = -1;
            A(T*(T-1)+i,3*T+i) = -1;
            
            b_U(T*(T-1)+i) = -tol;
            
            % Second MIP inequality:
            % -u_t + u_s + X_ts <= 1
            
            A(2*T*(T-1)+i,T+t) = -1;
            A(2*T*(T-1)+i,T+s) = 1;
            A(2*T*(T-1)+i,3*T+i) = 1;
            
            b_U(2*T*(T-1)+i) = 1;
            
            % Third MIP inequality:
            % l_t*z_t*(w_t-w_s) + V_t - V_s - X_ts*A_t < 0 
            
            A(3*T*(T-1)+i,2*T+t) = zw(t,s);
            A(3*T*(T-1)+i,t) = 1;
            A(3*T*(T-1)+i,s) = -1;
            A(3*T*(T-1)+i,3*T+i) = -Avec(t);
            
            b_U(3*T*(T-1)+i) = -tol;
            
            % Fourth MIP inequality:
            % -V_t + V_s + l_s*z_s*(w_s-w_t) + X_ts*A_s <= A_s
            
            A(4*T*(T-1)+i,2*T+s) = zw(s,t);
            A(4*T*(T-1)+i,t) = -1;
            A(4*T*(T-1)+i,s) = 1;
            A(4*T*(T-1)+i,3*T+i) = Avec(s);
            
            b_U(4*T*(T-1)+i) = Avec(s);
            
        end; % (s~=t)
    end; % (s)    
end; % (t)

% objective function
		
c = zeros(NPAR,1);

% upper and lower bounds on parameters:
% 0 <= V_t <= 1-tol
% 0 <= u_t <= 1-tol
% tol <= l_t <= 1
% X_ts = {0,1}

x_L = [zeros(2*T,1);tol*ones(T,1);zeros(T*(T-1),1)];
x_U = [(1-tol)*ones(2*T,1);ones(T,1);ones(T*(T-1),1)];

% SOLVE MIP PROBLEM

cpxControl = [];
callback   = [];
PriLev     = 0;
Prob       = 1;
PI         = [];
SC         = [];
SI         = [];
sos1       = [];
sos2       = [];
IntVars    = [zeros(3*T,1);ones(T*(T-1),1)];            % specifies that the first 3T parameters aee continuous and the T*(T-1) rest are binary

%% Calling the solver
Prob = mipAssign(c,A,b_L,b_U,x_L,x_U,[],[],[],[],IntVars,...
    [],[],[],[],[],[],[],[],[]);

Prob = ProbCheck(Prob, 'cplex');
Result = tomRun('cplex', Prob, 0);

disp(Result.ExitText);

    if isequal(Result.ExitFlag,0) %0: Global optimal solution found
        pass = 1;
   
    elseif ismember(Result.ExitFlag,4) % 4: No integer sol found
         %general eq model is not rationalizable
        pass = 0;
 
    elseif ismember(Result.Exitflag,[1 2 5]) % 1: Max nr of iterations reached 2: unbounded 5: some error
       %No answer for household
        pass = 2;
    else
        error('Tomlab solver gives unexpected ExitFlag %g!',Result.Exitflag)
    end

