% Implements weak sep test by Cherchye, L. T. Demuynck, B. De Rock and P. Hjertstrand (2015) 
% "Revealed preference tests for weak separability: an integer programming approach"
% Journal of Econometrics 186, 129-141. 
%
% This procedure assumes that the quantities are contaminated with errors
% and calculates the minimal standard deviation such that weak separability
% is satisfied.
%
% NOTE: Requires TOMLAB/CPLEX. See http://tomopt.com/tomlab/products/cplex/

clear all; clc;

warning('off','all');
warning;

load HSWdata;                                   % load data

s = 0.00001;                                    % standard deviation for measurement errors

x = HSWdata(1:47,2:18);                         % quantity data
p = HSWdata(48:94,2:18);                        % full price data

%sep = [5 6 7 8 9 10 11 12 13];                 % S6
%sep = [5 6 7 8 9 10 11 12 13 14 17];           % S7
%sep = [5 6 7 8 9 10 11 14];                    % S8   
%sep = [1 2 3];                                 % S9

alfa = 0.05;                                % nominal significance level
B = 10000;                                  % # simulations to calculate critical value

disp('Separable goods');                    % shows separable goods
disp(sep);

% SEPARABLE GROUP

r = p(:,sep);                               % prices in separable group
y = x(:,sep);                               % quantities in separable group
KSEP = size(r,2);         % # goods in separable group

% NON-SEPARABLE GROUP

[T K] = size(p);                            % size of data, T: #obs and K: #goods

NUMG = (1:1:(1+1*(K-1)))';                  % sequence of numbers up to K

nsep = NUMG';
nsep(sep)=[];                               % goods in non-separable group

z = p(:,nsep);                              % prices in non-separable group
w = x(:,nsep);                              % quantities in non-separable group
KNSEP = size(z,2);          % # goods in non-separable group

F = FminWeakSep(p,x,sep);                   % calculates F

% Calculating critical value and testing the hypothesis

h = 0;

for b = 1:B;
           
    e_sep = s*randn(T,KSEP).*y;             % generating normal errors for separable group
    e_nsep = s*randn(T,KNSEP).*w;           % generating normal errors for non-separable group

    re = zeros(T,T);            % cross-error matrix for separable group
    ze = zeros(T,T);            % cross-error matrix for non-separable group
        
    for t=1:T;
        for s=1:T;  
            re(t,s) = sum((r(t,:).*(e_sep(s,:)-e_sep(t,:)))');
            ze(t,s) = sum((z(t,:).*(e_nsep(s,:)-e_nsep(t,:)))');
        end;
    end;
    
    m = max([max(max(re)) max(max(re))]); 
    
    h = h + (F > m)/B;
    
end;
    
C = h > (1-alfa);               % C = 1 if reject and 0 otherwise

